# coding: utf8

__title__ = 'scrapydweb'
__version__ = '1.0.0rc2'
__author__ = 'my8100'
__author_email__ = 'my8100@gmail.com'
__url__ = 'https://github.com/my8100/scrapydweb'
__license__ = 'GNU General Public License v3.0'
__description__ = ("A full-featured web UI for Scrapyd cluster management, "
                   "with Scrapy log analysis & visualization supported.")
