/* loader */
/* https://www.w3schools.com/howto/howto_css_loader.asp */
/* https://stackoverflow.com/questions/6256043/css-position-loading-indicator-in-the-center-of-the-screen */


/* https://githubtimeline.xyz */
.loader {
    z-index:99;
    cursor: pointer;
    position: fixed;
    left: 50%;
    top: 50%;

    border: 5px solid #f3f3f3; /* Light grey */
    border-top: 5px solid #3498db; /* Blue */
    border-radius: 50%;
    width: 3em;
    height: 3em;
    animation: spin 1s linear infinite;
    display: none;
}

@keyframes spin {
    0% { transform: rotate(0deg); }
    100% { transform: rotate(360deg); }
}