#content pre {
white-space: pre-wrap;
/*word-wrap: break-word;*/
}

#content .tab-content .table {
  border: 1px solid #d8d8d8;
}

/* commented by llx --end */

/* #content a:hover { */
  /* color: #409EFF; */
/* } */

/* commented by llx --end */

#content a.icon {
  /*display: flex;*/
  width: 50px;
  height: 30px;
  font-size: 20px;
  color: #707070;
/*  justify-content: center;
  align-items: center;*/
}

#content .icon-back {
  width: 100%;
  height: 100%;
}

#content .icon-back:before {
  display: inline-block;
  width: 60px;
  height: 30px;
  font-size: 20px;
  line-height: 30px;
  text-align: center;
  border-radius: 3px;
  background: #409EFF;
  color: #fff;
  opacity: 0.7;
  transition: .1s all ease-in-out;
}

#content .icon-back:hover:before {
  color: #409EFF;
  border-color: #409EFF;
  opacity: 1;
  color: #fff;
}
